@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2021 Douglas Gilbert.
+ * Copyright (c) 2016-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -35,7 +35,7 @@
  * device. Based on sbc4r10.pdf .
  */
 
-static const char * version_str = "1.12 20210830";
+static const char * version_str = "1.15 20230618";
 
 #define BACKGROUND_CONTROL_SA 0x15
 
@@ -45,13 +45,13 @@ static const char * version_str = "1.12 20210830";
 static const char * cmd_name = "Background control";
 
 
-static struct option long_options[] = {
-        {"ctl", required_argument, 0, 'c'},
-        {"help", no_argument, 0, 'h'},
-        {"time", required_argument, 0, 't'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const struct option long_options[] = {
+    {"ctl", required_argument, 0, 'c'},
+    {"help", no_argument, 0, 'h'},
+    {"time", required_argument, 0, 't'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 
@@ -93,7 +93,7 @@ sg_ll_background_control(int sg_fd, unsigned int bo_ctl, unsigned int bo_time,
     uint8_t bcCDB[16] = {SG_SERVICE_ACTION_IN_16,
            BACKGROUND_CONTROL_SA, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,
            0, 0, 0, 0};
-    uint8_t sense_b[SENSE_BUFF_LEN];
+    uint8_t sense_b[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_pt_base * ptvp;
 
     if (bo_ctl)
@@ -208,7 +208,7 @@ main(int argc, char * argv[])
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false; */
         version_given = false;
         verbose = 0;
     } else if (! verbose_given) {
