@@ -1,17 +1,20 @@
-/* A utility program for the Linux OS SCSI generic ("sg") device driver.
-*  Copyright (C) 2001 - 2018 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
+/*
+ *  A utility program for the Linux OS SCSI generic ("sg") device driver.
+ *    Copyright (C) 2001 - 2023 D. Gilbert
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
 
    This program reads data from the given SCSI device (typically a disk
    or cdrom) and discards that data. Its primary goal is to time
    multiple reads all starting from the same logical address. Its interface
    is a subset of another member of this package: sg_dd which is a
    "dd" variant. The input file can be a scsi generic device, a block device,
-   a raw device or a seekable file. Streams such as stdin are not acceptable.
-   The block size ('bs') is assumed to be 512 if not given.
+   or a seekable file. Streams such as stdin are not acceptable. The block
+   size ('bs') is assumed to be 512 if not given.
 
    This version should compile with Linux sg drivers with version numbers
    >= 30000 . For mmap-ed IO the sg version number >= 30122 .
@@ -38,27 +41,36 @@
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/sysmacros.h>
-#include <sys/types.h>  /* needed for lseek64() */
+#ifndef major
+#include <sys/types.h>
+#endif
 #include <sys/mman.h>
 #include <sys/time.h>
-#include <linux/major.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef HAVE_LINUX_MAJOR_H
+#include <linux/major.h>
+#else
+#include "sg_pt_linux_missing.h"
+#endif
+
 #include "sg_lib.h"
 #include "sg_io_linux.h"
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "1.30 20180219";
+static const char * version_str = "1.40 20230716";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
 #define DEF_SCSI_CDBSZ 10
 #define MAX_SCSI_CDBSZ 16
+#define MAX_BPT_VALUE (1 << 24)         /* used for maximum bs as well */
+#define MAX_COUNT_SKIP_SEEK (1LL << 48) /* coverity wants upper bound */
 
 #define ME "sg_read: "
 
@@ -70,10 +82,10 @@ static const char * version_str = "1.30 20180219";
 #define DEF_TIMEOUT 40000       /* 40,000 millisecs == 40 seconds */
 
 #ifndef RAW_MAJOR
-#define RAW_MAJOR 255   /*unlikey value */
+#define RAW_MAJOR 255   /*unlikely value */
 #endif
 
-#define FT_OTHER 1              /* filetype other than sg or raw device */
+#define FT_OTHER 1              /* filetype other than sg and ... */
 #define FT_SG 2                 /* filetype is sg char device */
 #define FT_RAW 4                /* filetype is raw char device */
 #define FT_BLOCK 8              /* filetype is block device */
@@ -91,9 +103,11 @@ static int in_partial = 0;
 static int pack_id_count = 0;
 static int verbose = 0;
 
-static const char * proc_allow_dio = "/proc/scsi/sg/allow_dio";
+static const char * sg_allow_dio = "/sys/module/sg/parameters/allow_dio";
 
-static void install_handler (int sig_num, void (*sig_handler) (int sig))
+
+static void
+install_handler (int sig_num, void (*sig_handler) (int sig))
 {
     struct sigaction sigact;
 
@@ -106,7 +120,8 @@ static void install_handler (int sig_num, void (*sig_handler) (int sig))
     }
 }
 
-static void print_stats(int iters, const char * str)
+static void
+print_stats(int iters, const char * str)
 {
     if (orig_count > 0) {
         if (0 != dd_count)
@@ -121,7 +136,8 @@ static void print_stats(int iters, const char * str)
         pr2serr("%s commands issued: %d\n", (str ? str : ""), iters);
 }
 
-static void interrupt_handler(int sig)
+static void
+interrupt_handler(int sig)
 {
     struct sigaction sigact;
 
@@ -134,14 +150,16 @@ static void interrupt_handler(int sig)
     kill (getpid (), sig);
 }
 
-static void siginfo_handler(int sig)
+static void
+siginfo_handler(int sig)
 {
     if (sig) { ; }      /* unused, dummy to suppress warning */
     pr2serr("Progress report, continuing ...\n");
     print_stats(0, NULL);
 }
 
-static int dd_filetype(const char * filename)
+static int
+dd_filetype(const char * filename)
 {
     struct stat st;
 
@@ -157,7 +175,8 @@ static int dd_filetype(const char * filename)
     return FT_OTHER;
 }
 
-static void usage()
+static void
+usage()
 {
     pr2serr("Usage: sg_read  [blk_sgio=0|1] [bpt=BPT] [bs=BS] "
             "[cdbsz=6|10|12|16]\n"
@@ -166,7 +185,8 @@ static void usage()
             "                [mmap=0|1] [no_dfxer=0|1] [odir=0|1] "
             "[skip=SKIP]\n"
             "                [time=TI] [verbose=VERB] [--help] "
-            "[--version]\n"
+            "[--verbose]\n"
+            "                [--version] "
             "  where:\n"
             "    blk_sgio 0->normal IO for block devices, 1->SCSI commands "
             "via SG_IO\n"
@@ -187,7 +207,7 @@ static void usage()
             "    fua      1-> set force unit access (FUA) in SCSI READs\n"
             "    if       an sg, block or raw device, or a seekable file (not "
             "stdin)\n"
-            "    mmap     1->perform mmaped IO on sg device, 0->indirect IO "
+            "    mmap     1->perform mmap-ed IO on sg device, 0->indirect IO "
             "(def)\n"
             "    no_dxfer 1->DMA to kernel buffers only, not user space, "
             "0->normal(def)\n"
@@ -197,15 +217,16 @@ static void usage()
             "    time     0->do nothing(def), 1->time from 1st cmd, 2->time "
             "from 2nd, ...\n"
             "    verbose  increase level of verbosity (def: 0)\n"
-            "    --help   print this usage message then exit\n"
-            "    --version  print version number then exit\n\n"
+            "    --help|-h    print this usage message then exit\n"
+            "    --verbose|-v   increase level of verbosity (def: 0)\n"
+            "    --version|-V   print version number then exit\n\n"
             "Issue SCSI READ commands, each starting from the same logical "
             "block address\n");
 }
 
-static int sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz,
-                             unsigned int blocks, int64_t start_block,
-                             bool write_true, bool fua, bool dpo)
+static int
+sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz, unsigned int blocks,
+                  int64_t start_block, bool write_true, bool fua, bool dpo)
 {
     int sz_ind;
     int rd_opcode[] = {0x8, 0x28, 0xa8, 0x88};
@@ -276,14 +297,13 @@ static int sg_build_scsi_cdb(uint8_t * cdbp, int cdb_sz,
 /* -3 medium/hardware error, -2 -> not ready, 0 -> successful,
    1 -> recoverable (ENOMEM), 2 -> try again (e.g. unit attention),
    3 -> try again (e.g. aborted command), -1 -> other unrecoverable error */
-static int sg_bread(int sg_fd, uint8_t * buff, int blocks,
-                    int64_t from_block, int bs, int cdbsz,
-                    bool fua, bool dpo, bool * diop, bool do_mmap,
-                    bool no_dxfer)
+static int
+sg_bread(int sg_fd, uint8_t * buff, int blocks, int64_t from_block, int bs,
+         int cdbsz, bool fua, bool dpo, bool * diop, bool do_mmap,
+         bool no_dxfer)
 {
-    int k;
     uint8_t rdCmd[MAX_SCSI_CDBSZ];
-    uint8_t senseBuff[SENSE_BUFF_LEN];
+    uint8_t senseBuff[SENSE_BUFF_LEN] SG_C_CPP_ZERO_INIT;
     struct sg_io_hdr io_hdr;
 
     if (sg_build_scsi_cdb(rdCmd, cdbsz, blocks, from_block, false, fua,
@@ -315,10 +335,10 @@ static int sg_bread(int sg_fd, uint8_t * buff, int blocks,
     io_hdr.timeout = DEF_TIMEOUT;
     io_hdr.pack_id = pack_id_count++;
     if (verbose > 1) {
-        pr2serr( "    read cdb: ");
-        for (k = 0; k < cdbsz; ++k)
-            pr2serr( "%02x ", rdCmd[k]);
-        pr2serr( "\n");
+        char b[128];
+
+        pr2serr("    READ cdb: %s\n",
+                sg_get_command_str(rdCmd, cdbsz, false, sizeof(b), b));
     }
 
     if (ioctl(sg_fd, SG_IO, &io_hdr) < 0) {
@@ -366,12 +386,27 @@ static int sg_bread(int sg_fd, uint8_t * buff, int blocks,
     return 0;
 }
 
+/* Returns the number of times 'ch' is found in string 's' given the
+ * string's length. */
+static int
+num_chs_in_str(const char * s, int slen, int ch)
+{
+    int res = 0;
+
+    while (--slen >= 0) {
+        if (ch == s[slen])
+            ++res;
+    }
+    return res;
+}
+
 #define STR_SZ 1024
 #define INF_SZ 512
-#define EBUFF_SZ 512
+#define EBUFF_SZ 768
 
 
-int main(int argc, char * argv[])
+int
+main(int argc, char * argv[])
 {
     bool count_given = false;
     bool dio_tmp;
@@ -382,6 +417,8 @@ int main(int argc, char * argv[])
     bool dpo = false;
     bool fua = false;
     bool no_dxfer = false;
+    bool verbose_given = false;
+    bool version_given = false;
     int bs = 0;
     int bpt = DEF_BLOCKS_PER_TRANSFER;
     int dio_incomplete = 0;
@@ -389,7 +426,8 @@ int main(int argc, char * argv[])
     int in_type = FT_OTHER;
     int ret = 0;
     int scsi_cdbsz = DEF_SCSI_CDBSZ;
-    int res, k, t, buf_sz, iters, infd, blocks, flags, blocks_per;
+    int res, k, t, buf_sz, iters, infd, blocks, flags, blocks_per, err;
+    int n, keylen;
     size_t psz;
     int64_t skip = 0;
     char * key;
@@ -420,34 +458,40 @@ int main(int argc, char * argv[])
             buf++;
         if (*buf)
             *buf++ = '\0';
+        keylen = strlen(key);
         if (0 == strcmp(key,"blk_sgio"))
             do_blk_sgio = !! sg_get_num(buf);
         else if (0 == strcmp(key,"bpt")) {
             bpt = sg_get_num(buf);
-            if (-1 == bpt) {
+            if ((bpt < 0) || (bpt > MAX_BPT_VALUE)) {
                 pr2serr( ME "bad argument to 'bpt'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"bs")) {
             bs = sg_get_num(buf);
-            if (-1 == bs) {
+            if ((bs < 0) || (bs > MAX_BPT_VALUE)) {
                 pr2serr( ME "bad argument to 'bs'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
-        } else if (0 == strcmp(key,"cdbsz"))
+        } else if (0 == strcmp(key,"cdbsz")) {
             scsi_cdbsz = sg_get_num(buf);
-        else if (0 == strcmp(key,"count")) {
+            if ((scsi_cdbsz < 0) || (scsi_cdbsz > 32)) {
+                pr2serr( ME "bad argument to 'cdbsz', expect 6, 10, 12, 16 "
+                        "or 32\n");
+                return SG_LIB_SYNTAX_ERROR;
+            }
+        } else if (0 == strcmp(key,"count")) {
             count_given = true;
             if ('-' == *buf) {
                 dd_count = sg_get_llnum(buf + 1);
-                if (-1 == dd_count) {
+                if ((dd_count < 0) || (dd_count > MAX_COUNT_SKIP_SEEK)) {
                     pr2serr( ME "bad argument to 'count'\n");
                     return SG_LIB_SYNTAX_ERROR;
                 }
                 dd_count = - dd_count;
             } else {
                 dd_count = sg_get_llnum(buf);
-                if (-1 == dd_count) {
+                if ((dd_count < 0) || (dd_count > MAX_COUNT_SKIP_SEEK)) {
                     pr2serr( ME "bad argument to 'count'\n");
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -458,38 +502,86 @@ int main(int argc, char * argv[])
             dpo = !! sg_get_num(buf);
         else if (0 == strcmp(key,"fua"))
             fua = !! sg_get_num(buf);
-        else if (strcmp(key,"if") == 0)
-            strncpy(inf, buf, INF_SZ);
-        else if (0 == strcmp(key,"mmap"))
+        else if (strcmp(key,"if") == 0) {
+            memcpy(inf, buf, INF_SZ - 1);
+            inf[INF_SZ - 1] = '\0';
+        } else if (0 == strcmp(key,"mmap"))
             do_mmap = !! sg_get_num(buf);
         else if (0 == strcmp(key,"no_dxfer"))
             no_dxfer = !! sg_get_num(buf);
         else if (0 == strcmp(key,"odir"))
             do_odir = !! sg_get_num(buf);
-        else if (strcmp(key,"of") == 0)
-            strncpy(outf, buf, INF_SZ);
-        else if (0 == strcmp(key,"skip")) {
+        else if (strcmp(key,"of") == 0) {
+            memcpy(outf, buf, INF_SZ - 1);
+            outf[INF_SZ - 1] = '\0';
+        } else if (0 == strcmp(key,"skip")) {
             skip = sg_get_llnum(buf);
-            if (-1 == skip) {
+            if ((skip < 0) || (skip > MAX_COUNT_SKIP_SEEK)) {
                 pr2serr( ME "bad argument to 'skip'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"time"))
             do_time = sg_get_num(buf);
-        else if (0 == strncmp(key, "verb", 4))
+        else if (0 == strncmp(key, "verb", 4)) {
+            verbose_given = true;
             verbose = sg_get_num(buf);
-        else if (0 == strncmp(key, "--help", 6)) {
+        } else if (0 == strncmp(key, "--help", 6)) {
             usage();
             return 0;
-        } else if (0 == strncmp(key, "--vers", 6)) {
-            pr2serr( ME ": %s\n", version_str);
-            return 0;
+        } else if (0 == strncmp(key, "--verb", 6)) {
+            verbose_given = true;
+            ++verbose;
+        } else if (0 == strncmp(key, "--vers", 6))
+            version_given = true;
+        else if ((keylen > 1) && ('-' == key[0]) && ('-' != key[1])) {
+            res = 0;
+            n = num_chs_in_str(key + 1, keylen - 1, 'h');
+            if (n > 0) {
+                usage();
+                return 0;
+            }
+            n = num_chs_in_str(key + 1, keylen - 1, 'v');
+            if (n > 0)
+                verbose_given = true;
+            verbose += n;
+            res += n;
+            n = num_chs_in_str(key + 1, keylen - 1, 'V');
+            if (n > 0)
+                version_given = true;
+            res += n;
+            if (res < (keylen - 1)) {
+                pr2serr("Unrecognised short option in '%s', try '--help'\n",
+                        key);
+                return SG_LIB_SYNTAX_ERROR;
+            }
         } else {
             pr2serr( "Unrecognized argument '%s'\n", key);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+
+#ifdef DEBUG
+    pr2serr("In DEBUG mode, ");
+    if (verbose_given && version_given) {
+        pr2serr("but override: '-vV' given, zero verbose and continue\n");
+        /* verbose_given = false; */
+        version_given = false;
+        verbose = 0;
+    } else if (! verbose_given) {
+        pr2serr("set '-vv'\n");
+        verbose = 2;
+    } else
+        pr2serr("keep verbose=%d\n", verbose);
+#else
+    if (verbose_given && version_given)
+        pr2serr("Not in DEBUG mode, so '-vV' has no special action\n");
+#endif
+    if (version_given) {
+        pr2serr( ME ": %s\n", version_str);
+        return 0;
+    }
+
     if (bs <= 0) {
         bs = DEF_BLOCK_SIZE;
         if ((dd_count > 0) && (bpt > 0))
@@ -515,11 +607,11 @@ int main(int argc, char * argv[])
     }
     if (do_dio && do_mmap) {
         pr2serr("cannot select both dio and mmap\n");
-        return SG_LIB_SYNTAX_ERROR;
+        return SG_LIB_CONTRADICT;
     }
     if (no_dxfer && (do_dio || do_mmap)) {
         pr2serr("cannot select no_dxfer with dio or mmap\n");
-        return SG_LIB_SYNTAX_ERROR;
+        return SG_LIB_CONTRADICT;
     }
 
     install_handler (SIGINT, interrupt_handler);
@@ -557,10 +649,11 @@ int main(int argc, char * argv[])
             if (do_odir)
                 flags |= O_DIRECT;
             if ((infd = open(inf, flags)) < 0) {
+                err = errno;
                 snprintf(ebuff, EBUFF_SZ,
                          ME "could not open %s for sg reading", inf);
                 perror(ebuff);
-                return SG_LIB_FILE_ERROR;
+                return sg_convert_errno(err);
             }
         }
         if (verbose)
@@ -599,10 +692,11 @@ int main(int argc, char * argv[])
         if (do_odir)
             flags |= O_DIRECT;
         if ((infd = open(inf, flags)) < 0) {
+            err = errno;
             snprintf(ebuff,  EBUFF_SZ,
                      ME "could not open %s for reading", inf);
             perror(ebuff);
-            return SG_LIB_FILE_ERROR;
+            return sg_convert_errno(err);
         }
         if (verbose)
             pr2serr("Opened %s for Unix reads with flags=0x%x\n", inf, flags);
@@ -611,10 +705,11 @@ int main(int argc, char * argv[])
 
             offset *= bs;       /* could exceed 32 bits here! */
             if (lseek64(infd, offset, SEEK_SET) < 0) {
+                err = errno;
                 snprintf(ebuff,  EBUFF_SZ,
                     ME "couldn't skip to required position on %s", inf);
                 perror(ebuff);
-                return SG_LIB_FILE_ERROR;
+                return sg_convert_errno(err);
             }
         }
     }
@@ -826,11 +921,11 @@ int main(int argc, char * argv[])
 
         pr2serr(">> Direct IO requested but incomplete %d times\n",
                 dio_incomplete);
-        if ((fd = open(proc_allow_dio, O_RDONLY)) >= 0) {
+        if ((fd = open(sg_allow_dio, O_RDONLY)) >= 0) {
             if (1 == read(fd, &c, 1)) {
                 if ('0' == c)
                     pr2serr(">>> %s set to '0' but should be set to '1' for "
-                            "direct IO\n", proc_allow_dio);
+                            "direct IO\n", sg_allow_dio);
             }
             close(fd);
         }
