@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999-2020 D. Gilbert
+ *  Copyright (C) 1999-2023 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -36,26 +36,28 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "0.67 20200930";  /* sbc3r14; mmc6r01a */
+static const char * version_str = "0.68 20230407";  /* sbc3r14; mmc6r01a */
 
-static struct option long_options[] = {
-        {"eject", no_argument, 0, 'e'},
-        {"fl", required_argument, 0, 'f'},
-        {"help", no_argument, 0, 'h'},
-        {"immed", no_argument, 0, 'i'},
-        {"load", no_argument, 0, 'l'},
-        {"loej", no_argument, 0, 'L'},
-        {"mod", required_argument, 0, 'm'},
-        {"noflush", no_argument, 0, 'n'},
-        {"new", no_argument, 0, 'N'},
-        {"old", no_argument, 0, 'O'},
-        {"pc", required_argument, 0, 'p'},
-        {"readonly", no_argument, 0, 'r'},
-        {"start", no_argument, 0, 's'},
-        {"stop", no_argument, 0, 'S'},
-        {"verbose", no_argument, 0, 'v'},
-        {"version", no_argument, 0, 'V'},
-        {0, 0, 0, 0},
+static const char * my_name = "sg_start: ";
+
+static const struct option long_options[] = {
+    {"eject", no_argument, 0, 'e'},
+    {"fl", required_argument, 0, 'f'},
+    {"help", no_argument, 0, 'h'},
+    {"immed", no_argument, 0, 'i'},
+    {"load", no_argument, 0, 'l'},
+    {"loej", no_argument, 0, 'L'},
+    {"mod", required_argument, 0, 'm'},
+    {"noflush", no_argument, 0, 'n'},
+    {"new", no_argument, 0, 'N'},
+    {"old", no_argument, 0, 'O'},
+    {"pc", required_argument, 0, 'p'},
+    {"readonly", no_argument, 0, 'r'},
+    {"start", no_argument, 0, 's'},
+    {"stop", no_argument, 0, 'S'},
+    {"verbose", no_argument, 0, 'v'},
+    {"version", no_argument, 0, 'V'},
+    {0, 0, 0, 0},
 };
 
 struct opts_t {
@@ -496,6 +498,8 @@ main(int argc, char * argv[])
     op = &opts;
     memset(op, 0, sizeof(opts));
     op->do_fl = -1;    /* only when >= 0 set FL bit */
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
     res = parse_cmd_line(op, argc, argv);
     if (res)
         return res;
