@@ -1,14 +1,17 @@
 /*
- * Copyright (c) 2006-2018 Douglas Gilbert.
+ * Copyright (c) 2006-2023 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
+ *
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #define __STDC_FORMAT_MACROS 1
 #include <inttypes.h>
@@ -25,13 +28,15 @@
 #include "sg_unaligned.h"
 #include "sg_pr2serr.h"
 
+#include "sg_vpd_common.h"
+
 /* This is a companion file to sg_vpd.c . It contains logic to output and
    decode vendor specific VPD pages
 
    This program fetches Vital Product Data (VPD) pages from the given
    device and outputs it as directed. VPD pages are obtained via a
    SCSI INQUIRY command. Most of the data in this program is obtained
-   from the SCSI SPC-4 document at http://www.t10.org .
+   from the SCSI SPC-4 document at https://www.t10.org .
 
    Acknowledgments:
       - Lars Marowsky-Bree <lmb at suse dot de> contributed Unit Path Report
@@ -51,9 +56,13 @@
 #define VPD_VP_HP3PAR 4
 #define VPD_VP_IBM_LTO 5
 #define VPD_VP_HP_LTO 6
+#define VPD_VP_WDC_HITACHI 7
+#define VPD_VP_NVME 8
+#define VPD_VP_SG 9     /* this package/library as a vendor */
 
 
 /* vendor VPD pages */
+#define VPD_V_HIT_PG3 0x3
 #define VPD_V_HP3PAR 0xc0
 #define VPD_V_FIRM_SEA  0xc0
 #define VPD_V_UPR_EMC  0xc0
@@ -80,138 +89,18 @@
 #define VPD_V_VAC_RDAC 0xc9
 #define VPD_V_RVSI_RDAC 0xca
 #define VPD_V_SAID_RDAC 0xd0
+#define VPD_V_HIT_PG_D1 0xd1
+#define VPD_V_HIT_PG_D2 0xd2
 
 
 #define DEF_ALLOC_LEN 252
 #define MX_ALLOC_LEN (0xc000 + 0x80)
 
-/* These structures are duplicates of those of the same name in
- * sg_vpd.c . Take care that both are the same. */
-
-struct opts_t {
-    bool do_all;
-    bool do_enum;
-    bool do_force;
-    bool do_long;
-    bool do_quiet;
-    int do_hex;
-    int vpd_pn;
-    int do_ident;
-    int maxlen;
-    int do_raw;
-    int vend_prod_num;
-    int verbose;
-    const char * device_name;
-    const char * page_str;
-    const char * inhex_fn;
-    const char * vend_prod;
-};
-
-struct svpd_values_name_t {
-    int value;       /* VPD page number */
-    int subvalue;    /* to differentiate if value+pdt are not unique */
-    int pdt;         /* peripheral device type id, -1 is the default */
-                     /* (all or not applicable) value */
-    const char * acron;
-    const char * name;
-};
-
-int vpd_fetch_page(int sg_fd, uint8_t * rp, int page, int mxlen,
-                   bool qt, int vb, int * rlenp);
-
-/* sharing large global buffer, defined in sg_vpd.c */
-extern uint8_t * rsp_buff;
-
-/* end of section copied from sg_vpd.c . Maybe sg_vpd.h is needed */
-
-struct svpd_vp_name_t {
-    int vend_prod_num;       /* vendor/product identifier */
-    const char * acron;
-    const char * name;
-};
-
-
-/* Supported vendor specific VPD pages */
-/* Arrange in alphabetical order by acronym */
-static struct svpd_vp_name_t vp_arr[] = {
-    {VPD_VP_DDS, "dds", "DDS tape family from IBM"},
-    {VPD_VP_EMC, "emc", "EMC (company)"},
-    {VPD_VP_HP3PAR, "hp3par", "3PAR array (HP was Left Hand)"},
-    {VPD_VP_IBM_LTO, "ibm_lto", "IBM LTO tape/systems"},
-    {VPD_VP_HP_LTO, "hp_lto", "HP LTO tape/systems"},
-    {VPD_VP_RDAC, "rdac", "RDAC array (NetApp E-Series)"},
-    {VPD_VP_SEAGATE, "sea", "Seagate disk"},
-    {0, NULL, NULL},
-};
-
-/* Supported vendor specific VPD pages */
-/* 'subvalue' holds vendor/product number to disambiguate */
-/* Arrange in alphabetical order by acronym */
-static struct svpd_values_name_t vendor_vpd_pg[] = {
-    {VPD_V_ACI_LTO, VPD_VP_HP_LTO, 1, "aci", "ACI revision level (HP LTO)"},
-    {VPD_V_DATC_SEA, VPD_VP_SEAGATE, 0, "datc", "Date code (Seagate)"},
-    {VPD_V_DCRL_LTO, VPD_VP_IBM_LTO, 1, "dcrl", "Drive component revision "
-     "levels (IBM LTO)"},
-    {VPD_V_FVER_DDS, VPD_VP_DDS, 1, "ddsver", "Firmware revision (DDS)"},
-    {VPD_V_DEV_BEH_SEA, VPD_VP_SEAGATE, 0, "devb", "Device behavior "
-     "(Seagate)"},
-    {VPD_V_DSN_LTO, VPD_VP_IBM_LTO, 1, "dsn", "Drive serial numbers (IBM "
-     "LTO)"},
-    {VPD_V_DUCD_LTO, VPD_VP_IBM_LTO, 1, "ducd", "Device unique "
-     "configuration data (IBM LTO)"},
-    {VPD_V_EDID_RDAC, VPD_VP_RDAC, 0, "edid", "Extended device "
-     "identification (RDAC)"},
-    {VPD_V_FEAT_RDAC, VPD_VP_RDAC, 0, "prm4", "Feature Parameters (RDAC)"},
-    {VPD_V_FIRM_SEA, VPD_VP_SEAGATE, 0, "firm", "Firmware numbers "
-     "(Seagate)"},
-    {VPD_V_FVER_LTO, VPD_VP_HP_LTO, 0, "frl", "Firmware revision level "
-     "(HP LTO)"},
-    {VPD_V_FVER_RDAC, VPD_VP_RDAC, 0, "fwr4", "Firmware version (RDAC)"},
-    {VPD_V_HEAD_LTO, VPD_VP_HP_LTO, 1, "head", "Head Assy revision level "
-     "(HP LTO)"},
-    {VPD_V_HP3PAR, VPD_VP_HP3PAR, 0, "hp3par", "Volume information "
-     "(HP/3PAR)"},
-    {VPD_V_HVER_LTO, VPD_VP_HP_LTO, 1, "hrl", "Hardware revision level "
-     "(HP LTO)"},
-    {VPD_V_HVER_RDAC, VPD_VP_RDAC, 0, "hwr4", "Hardware version (RDAC)"},
-    {VPD_V_JUMP_SEA, VPD_VP_SEAGATE, 0, "jump", "Jump setting (Seagate)"},
-    {VPD_V_MECH_LTO, VPD_VP_HP_LTO, 1, "mech", "Mechanism revision level "
-     "(HP LTO)"},
-    {VPD_V_MPDS_LTO, VPD_VP_IBM_LTO, 1, "mpds", "Mode parameter default "
-     "settings (IBM LTO)"},
-    {VPD_V_PCA_LTO, VPD_VP_HP_LTO, 1, "pca", "PCA revision level (HP LTO)"},
-    {VPD_V_RVSI_RDAC, VPD_VP_RDAC, 0, "rvsi", "Replicated volume source "
-     "identifier (RDAC)"},
-    {VPD_V_SAID_RDAC, VPD_VP_RDAC, 0, "said", "Storage array world wide "
-     "name (RDAC)"},
-    {VPD_V_SUBS_RDAC, VPD_VP_RDAC, 0, "subs", "Subsystem identifier (RDAC)"},
-    {VPD_V_SVER_RDAC, VPD_VP_RDAC, 0, "swr4", "Software version (RDAC)"},
-    {VPD_V_UPR_EMC, VPD_VP_EMC, 0, "upr", "Unit path report (EMC)"},
-    {VPD_V_VAC_RDAC, VPD_VP_RDAC, 0, "vac1", "Volume access control (RDAC)"},
-    {0, 0, 0, NULL, NULL},
-};
-
-
-void
-dup_sanity_chk(int sz_opts_t, int sz_values_name_t)
-{
-    const size_t my_sz_opts_t = sizeof(struct opts_t);
-    const size_t my_sz_values_name_t = sizeof(struct svpd_values_name_t);
-
-    if (sz_opts_t != (int)my_sz_opts_t)
-        pr2serr(">>> struct opts_t differs in size from sg_vpd.c [%d != "
-                "%d]\n", (int)my_sz_opts_t, sz_opts_t);
-    if (sz_values_name_t != (int)my_sz_values_name_t)
-        pr2serr(">>> struct svpd_values_name_t differs in size from "
-                "sg_vpd.c [%d != %d]\n", (int)my_sz_values_name_t,
-                sz_values_name_t);
-}
-
-static int
+static bool
 is_like_pdt(int actual_pdt, const struct svpd_values_name_t * vnp)
 {
     if (actual_pdt == vnp->pdt)
-        return 1;
+        return true;
     if (PDT_DISK == vnp->pdt) {
         switch (actual_pdt) {
         case PDT_DISK:
@@ -219,21 +108,21 @@ is_like_pdt(int actual_pdt, const struct svpd_values_name_t * vnp)
         case PDT_PROCESSOR:
         case PDT_SAC:
         case PDT_ZBC:
-            return 1;
+            return true;
         default:
-            return 0;
+            return false;
         }
     } else if (PDT_TAPE == vnp->pdt) {
         switch (actual_pdt) {
         case PDT_TAPE:
         case PDT_MCHANGER:
         case PDT_ADC:
-            return 1;
+            return true;
         default:
-            return 0;
+            return false;
         }
     } else
-        return 0;
+        return false;
 }
 
 static const struct svpd_values_name_t *
@@ -272,22 +161,6 @@ svpd_find_vendor_by_num(int page_num, int vend_prod_num)
     return NULL;
 }
 
-
-int
-svpd_find_vp_num_by_acron(const char * vp_ap)
-{
-    size_t len;
-    const struct svpd_vp_name_t * vpp;
-
-    for (vpp = vp_arr; vpp->acron; ++vpp) {
-        len = strlen(vpp->acron);
-        if (0 == strncmp(vpp->acron, vp_ap, len))
-            return vpp->vend_prod_num;
-    }
-    return -1;
-}
-
-
 const struct svpd_values_name_t *
 svpd_find_vendor_by_acron(const char * ap)
 {
@@ -300,43 +173,6 @@ svpd_find_vendor_by_acron(const char * ap)
     return NULL;
 }
 
-/* if vend_prod_num < -1 then list vendor_product ids + vendor pages, =-1
- * list only vendor_product ids, else list pages for that vend_prod_num */
-void
-svpd_enumerate_vendor(int vend_prod_num)
-{
-    bool seen;
-    const struct svpd_vp_name_t * vpp;
-    const struct svpd_values_name_t * vnp;
-
-    if (vend_prod_num < 0) {
-        for (seen = false, vpp = vp_arr; vpp->acron; ++vpp) {
-            if (vpp->name) {
-                if (! seen) {
-                    printf("\nVendor/product identifiers:\n");
-                    seen = true;
-                }
-                printf("  %-10s %d      %s\n", vpp->acron,
-                       vpp->vend_prod_num, vpp->name);
-            }
-        }
-    }
-    if (-1 == vend_prod_num)
-        return;
-    for (seen = false, vnp = vendor_vpd_pg; vnp->acron; ++vnp) {
-        if ((vend_prod_num >= 0) && (vend_prod_num != vnp->subvalue))
-            continue;
-        if (vnp->name) {
-            if (! seen) {
-                printf("\nVendor specific VPD pages:\n");
-                seen = true;
-            }
-            printf("  %-10s 0x%02x,%d      %s\n", vnp->acron,
-                   vnp->value, vnp->subvalue, vnp->name);
-        }
-    }
-}
-
 int
 svpd_count_vendor_vpds(int vpd_pn, int vend_prod_num)
 {
@@ -488,127 +324,6 @@ decode_dev_beh_vpd_c3_sea(uint8_t * buff, int len)
     printf("  Default number of cache segments: %d\n", buff[24]);
 }
 
-static const char * lun_state_arr[] =
-{
-    "LUN not bound or LUN_Z report",
-    "LUN bound, but not owned by this SP",
-    "LUN bound and owned by this SP",
-};
-
-static const char * ip_mgmt_arr[] =
-{
-    "No IP access",
-    "Reserved (undefined)",
-    "via IPv4",
-    "via IPv6",
-};
-
-static const char * sp_arr[] =
-{
-    "SP A",
-    "SP B",
-};
-
-static const char * lun_op_arr[] =
-{
-    "Normal operations",
-    "I/O Operations being rejected, SP reboot or NDU in progress",
-};
-
-static const char * failover_mode_arr[] =
-{
-    "Legacy mode 0",
-    "Unknown mode (1)",
-    "Unknown mode (2)",
-    "Unknown mode (3)",
-    "Active/Passive (PNR) mode 1",
-    "Unknown mode (5)",
-    "Active/Active (ALUA) mode 4",
-    "Unknown mode (7)",
-    "Legacy mode 2",
-    "Unknown mode (9)",
-    "Unknown mode (10)",
-    "Unknown mode (11)",
-    "Unknown mode (12)",
-    "Unknown mode (13)",
-    "AIX Active/Passive (PAR) mode 3",
-    "Unknown mode (15)",
-};
-
-static void
-decode_upr_vpd_c0_emc(uint8_t * buff, int len)
-{
-    int k, ip_mgmt, vpp80, lun_z;
-
-    if (len < 3) {
-        pr2serr("EMC upr VPD page [0xc0]: length too short=%d\n", len);
-        return;
-    }
-    if (buff[9] != 0x00) {
-        pr2serr("Unsupported page revision %d, decoding not possible.\n",
-                buff[9]);
-        return;
-    }
-    printf("  LUN WWN: ");
-    for (k = 0; k < 16; ++k)
-        printf("%02x", buff[10 + k]);
-    printf("\n");
-    printf("  Array Serial Number: ");
-    dStrRaw(&buff[50], buff[49]);
-    printf("\n");
-
-    printf("  LUN State: ");
-    if (buff[4] > 0x02)
-           printf("Unknown (%x)\n", buff[4]);
-    else
-           printf("%s\n", lun_state_arr[buff[4]]);
-
-    printf("  This path connects to: ");
-    if (buff[8] > 0x01)
-           printf("Unknown SP (%x)", buff[8]);
-    else
-           printf("%s", sp_arr[buff[8]]);
-    printf(", Port Number: %u\n", buff[7]);
-
-    printf("  Default Owner: ");
-    if (buff[5] > 0x01)
-           printf("Unknown (%x)\n", buff[5]);
-    else
-           printf("%s\n", sp_arr[buff[5]]);
-
-    printf("  NO_ATF: %s, Access Logix: %s\n",
-                   buff[6] & 0x80 ? "set" : "not set",
-                   buff[6] & 0x40 ? "supported" : "not supported");
-
-    ip_mgmt = (buff[6] >> 4) & 0x3;
-
-    printf("  SP IP Management Mode: %s\n", ip_mgmt_arr[ip_mgmt]);
-    if (ip_mgmt == 2)
-        printf("  SP IPv4 address: %u.%u.%u.%u\n",
-               buff[44], buff[45], buff[46], buff[47]);
-    else {
-        printf("  SP IPv6 address: ");
-        for (k = 0; k < 16; ++k)
-            printf("%02x", buff[32 + k]);
-        printf("\n");
-    }
-
-    vpp80 = buff[30] & 0x08;
-    lun_z = buff[30] & 0x04;
-
-    printf("  System Type: %x, Failover mode: %s\n",
-           buff[27], failover_mode_arr[buff[28] & 0x0f]);
-
-    printf("  Inquiry VPP 0x80 returns: %s, Arraycommpath: %s\n",
-                   vpp80 ? "array serial#" : "LUN serial#",
-                   lun_z ? "Set to 1" : "Unknown");
-
-    printf("  Lun operations: %s\n",
-               buff[48] > 1 ? "undefined" : lun_op_arr[buff[48]]);
-
-    return;
-}
-
 static void
 decode_rdac_vpd_c0(uint8_t * buff, int len)
 {
@@ -697,61 +412,6 @@ decode_rdac_vpd_c1(uint8_t * buff, int len)
     return;
 }
 
-static void
-decode_rdac_vpd_c2(uint8_t * buff, int len)
-{
-    int i, n, v, r, m, p, d, y, num_part;
-    char part[5];
-
-    if (len < 3) {
-        pr2serr("Software Version VPD page length too short=%d\n", len);
-        return;
-    }
-    if (buff[4] != 's' && buff[5] != 'w' && buff[6] != 'r') {
-        pr2serr("Invalid page identifier %c%c%c%c, decoding not possible.\n",
-                buff[4], buff[5], buff[6], buff[7]);
-        return;
-    }
-    printf("  Software Version: %02x.%02x.%02x\n", buff[8], buff[9], buff[10]);
-    printf("  Software Date: %02d/%02d/%02d\n", buff[11], buff[12], buff[13]);
-    printf("  Features:");
-    if (buff[14] & 0x01)
-        printf(" Dual Active,");
-    if (buff[14] & 0x02)
-        printf(" Series 3,");
-    if (buff[14] & 0x04)
-        printf(" Multiple Sub-enclosures,");
-    if (buff[14] & 0x08)
-        printf(" DCE/DRM/DSS/DVE,");
-    if (buff[14] & 0x10)
-        printf(" Asymmetric Logical Unit Access,");
-    printf("\n");
-    printf("  Max. #of LUNS: %d\n", buff[15]);
-
-    num_part = (len - 12) / 16;
-    n = 16;
-    printf("  Partitions: %d\n", num_part);
-    for (i = 0; i < num_part; i++) {
-        memset(part,0, 5);
-        memcpy(part, &buff[n], 4);
-        printf("    Name: %s\n", part);
-        n += 4;
-        v = buff[n++];
-        r = buff[n++];
-        m = buff[n++];
-        p = buff[n++];
-        printf("    Version: %d.%d.%d.%d\n", v, r, m, p);
-        m = buff[n++];
-        d = buff[n++];
-        y = buff[n++];
-        printf("    Date: %d/%d/%d\n", m, d, y);
-
-        n += 5;
-    }
-
-    return;
-}
-
 static void
 decode_rdac_vpd_c3(uint8_t * buff, int len)
 {
@@ -965,6 +625,7 @@ decode_rdac_vpd_c8(uint8_t * buff, int len)
     return;
 }
 
+#if 0
 static void
 decode_rdac_vpd_c9_rtpg_data(uint8_t aas, uint8_t vendor)
 {
@@ -1130,6 +791,7 @@ decode_rdac_vpd_c9(uint8_t * buff, int len)
         decode_rdac_vpd_c9_rtpg_data(buff[12], buff[13]);
     }
 }
+#endif
 
 static void
 decode_rdac_vpd_ca(uint8_t * buff, int len)
@@ -1297,18 +959,102 @@ decode_ibm_lto_dsn(uint8_t * buff, int len)
     printf("  Reported serial number: %.12s\n", buff + 16);
 }
 
+static void
+decode_vpd_3_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 184) || (blen < 184)) {
+        pr2serr("Hitachi VPD page 0x3 length (%u) shorter than %u\n",
+                plen + 4, 184 + 4);
+        return;
+    }
+    printf("  ASCII uCode Identifier: %.12s\n", b + 24);
+    printf("  ASCII servo P/N: %.4s\n", b + 36);
+    printf("  Major Version: %.2s\n", b + 40);
+    printf("  Minor Version: %.2s\n", b + 42);
+    printf("  User Count: %.4s\n", b + 44);
+    printf("  Build Number: %.4s\n", b + 48);
+    printf("  Build Date String: %.32s\n", b + 52);
+    printf("  Product ID: %.8s\n", b + 84);
+    printf("  Interface ID: %.8s\n", b + 92);
+    printf("  Code Type: %.8s\n", b + 100);
+    printf("  User Name: %.12s\n", b + 108);
+    printf("  Machine Name: %.16s\n", b + 120);
+    printf("  Directory Name: %.32s\n", b + 136);
+    printf("  Operating state: %u\n", sg_get_unaligned_be32(b + 168));
+    printf("  Functional Mode: %u\n", sg_get_unaligned_be32(b + 172));
+    printf("  Degraded Reason: %u\n", sg_get_unaligned_be32(b + 176));
+    printf("  Broken Reason: %u\n", sg_get_unaligned_be32(b + 180));
+    printf("  Code Mode: %u\n", sg_get_unaligned_be32(b + 184));
+    printf("  Revision: %.4s\n", b + 188);
+}
+
+static void
+decode_vpd_d1_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 80) || (blen < 80)) {
+        pr2serr("Hitachi VPD page 0xd1 length (%u) shorter than %u\n",
+                plen + 4, 80 + 4);
+        return;
+    }
+    printf("  ASCII Media Disk Definition: %.16s\n", b + 4);
+    printf("  ASCII Motor Serial Number: %.16s\n", b + 20);
+    printf("  ASCII Flex Assembly Serial Number: %.16s\n", b + 36);
+    printf("  ASCII Actuator Serial Number: %.16s\n", b + 52);
+    printf("  ASCII Device Enclosure Serial Number: %.16s\n", b + 68);
+}
+
+static void
+decode_vpd_d2_hit(uint8_t * b, int blen)
+{
+    uint16_t plen = sg_get_unaligned_be16(b + 2);
+
+    if ((plen < 52) || (blen < 52)) {
+        pr2serr("Hitachi VPD page 0xd2 length (%u) shorter than %u\n",
+                plen + 4, 52 + 4);
+        return;
+    }
+    if ((blen - 4) == 120) {
+        printf("  HDC Version: %.*s\n", b[4], b + 5);
+        printf("  Card Serial Number: %.*s\n", b[24], b + 25);
+        printf("  NAND Flash Version: %.*s\n", b[44], b + 45);
+        printf("  Card Assembly Part Number: %.*s\n", b[64], b + 65);
+        printf("  Second Card Serial Number: %.*s\n", b[84], b + 85);
+        printf("  Second Card Assembly Part Number: %.*s\n", b[104], b + 105);
+    } else {
+        printf("  ASCII HDC Version: %.16s\n", b + 5);
+        printf("  ASCII Card Serial Number: %.16s\n", b + 22);
+        printf("  ASCII Card Assembly Part Number: %.16s\n", b + 39);
+    }
+}
+
 /* Returns 0 if successful, see sg_ll_inquiry() plus SG_LIB_CAT_OTHER for
    unsupported page */
 int
-svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
+svpd_decode_vendor(struct sg_pt_base * ptvp, struct opts_t * op,
+		   sgj_opaque_p jop, int off)
 {
-    int len, res;
-    char name[64];
-    const struct svpd_values_name_t * vnp;
+    bool as_json;
+    int len, pdt, plen, pn, dhex;
     int alloc_len = op->maxlen;
+    int res = 0;
+    const struct svpd_values_name_t * vnp;
+    sgj_state * jsp = &op->json_st;
+    sgj_opaque_p jo2p = NULL;
     uint8_t * rp;
+    char name[80];
 
-    switch (op->vpd_pn) {
+    as_json = jsp->pr_as_json;
+    dhex = op->do_hex;
+    if (dhex < 0)
+	dhex = -dhex;
+    pn = op->vpd_pn;
+
+    switch (pn) {       /* VPD codes that we support vendor pages for */
+    case 0x3:
     case 0xc0:
     case 0xc1:
     case 0xc2:
@@ -1319,126 +1065,263 @@ svpd_decode_vendor(int sg_fd, struct opts_t * op, int off)
     case 0xc9:
     case 0xca:
     case 0xd0:
+    case 0xd1:
+    case 0xd2:
+    case 0xde:
         break;
     default:    /* not known so return prior to fetching page */
         return SG_LIB_CAT_OTHER;
     }
     rp = rsp_buff + off;
-    if (sg_fd >= 0) {
+    if (ptvp) {
         if (0 == alloc_len)
             alloc_len = DEF_ALLOC_LEN;
     }
-    res = vpd_fetch_page(sg_fd, rp, op->vpd_pn, alloc_len, op->do_quiet,
-                                  op->verbose, &len);
-    if (0 == res) {
-        vnp = svpd_get_v_detail(op->vpd_pn, op->vend_prod_num, 0xf & rp[0]);
-        if (vnp && vnp->name)
-            strcpy(name, vnp->name);
-        else
-            snprintf(name, sizeof(name) - 1, "Vendor VPD page=0x%x",
-                     op->vpd_pn);
-        if ((! op->do_raw) && (! op->do_quiet) && (op->do_hex < 2))
-            printf("%s VPD Page:\n", name);
-        if (op->do_raw)
-            dStrRaw(rp, len);
-        else if (op->do_hex)
-            hex2stdout(rp, len, ((1 == op->do_hex) ? 0 : -1));
-        else {
-            switch(op->vpd_pn) {
-                case 0xc0:
-                    if (VPD_VP_SEAGATE == op->vend_prod_num)
-                        decode_firm_vpd_c0_sea(rp, len);
-                    else if (VPD_VP_EMC == op->vend_prod_num)
-                        decode_upr_vpd_c0_emc(rp, len);
-                    else if (VPD_VP_HP3PAR == op->vend_prod_num)
-                        decode_vpd_c0_hp3par(rp, len);
-                    else if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c0(rp, len);
-                    else if (VPD_VP_DDS == op->vend_prod_num)
-                        decode_dds_vpd_c0(rp, len);
-                    else if (VPD_VP_IBM_LTO == op->vend_prod_num)
-                        decode_ibm_lto_dcrl(rp, len);
-                    else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc1:
-                    if (VPD_VP_SEAGATE == op->vend_prod_num)
-                        decode_date_code_vpd_c1_sea(rp, len);
-                    else if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c1(rp, len);
-                    else if (VPD_VP_IBM_LTO == op->vend_prod_num)
-                        decode_ibm_lto_dsn(rp, len);
-                    else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc2:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c2(rp, len);
-                    else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc3:
-                    if (VPD_VP_SEAGATE == op->vend_prod_num)
-                        decode_dev_beh_vpd_c3_sea(rp, len);
-                    else if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c3(rp, len);
-                    else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc4:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c4(rp, len);
-                    else if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc5:
-                    if (VPD_VP_HP_LTO == op->vend_prod_num)
-                        decode_hp_lto_vpd_cx(rp, len, op->vpd_pn);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc8:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c8(rp, len);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xc9:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_c9(rp, len);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xca:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_ca(rp, len);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                case 0xd0:
-                    if (VPD_VP_RDAC == op->vend_prod_num)
-                        decode_rdac_vpd_d0(rp, len);
-                    else
-                        hex2stdout(rp, len, 0);
-                    break;
-                default:
-                    pr2serr("%s: logic error, should know can't decode "
-                            "pn=0x%x\n", __func__, op->vpd_pn);
-                    return SG_LIB_CAT_OTHER;
+    res = vpd_fetch_page(ptvp, rp, pn, alloc_len, op->do_quiet, op->verbose,
+                         &len);
+    if (res) {
+        pr2serr("Vendor VPD page=0x%x  failed to fetch\n", pn);
+        return res;
+    }
+    pdt = rp[0] & PDT_MASK;
+    vnp = svpd_get_v_detail(pn, op->vend_prod_num, pdt);
+    if (vnp && vnp->name)
+        snprintf(name, sizeof(name), "%s", vnp->name);
+    else
+        snprintf(name, sizeof(name) - 1, "Vendor VPD page=0x%x", pn);
+    if ((! op->do_raw) && (! op->do_quiet) && (op->do_hex < 3))
+        sgj_pr_hr(jsp, "%s VPD Page:\n", name);
+    if (op->do_raw)
+        dStrRaw(rp, len);
+    else {
+        switch(pn) {
+        case 0x3:
+            if ((0 == dhex) && (VPD_VP_WDC_HITACHI == op->vend_prod_num))
+                decode_vpd_3_hit(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc0:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_SEAGATE == op->vend_prod_num)
+                decode_firm_vpd_c0_sea(rp, len);
+            else if (VPD_VP_EMC == op->vend_prod_num) {
+                if (as_json)
+                    jo2p = sg_vpd_js_hdr(jsp, jop,
+                                         "Unit serial number VPD page", rp);
+                decode_upr_vpd_c0_emc(rp, len, op, jo2p);
+            } else if (VPD_VP_HP3PAR == op->vend_prod_num)
+                decode_vpd_c0_hp3par(rp, len);
+            else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_c0(rp, len);
+            else if (VPD_VP_DDS == op->vend_prod_num)
+                decode_dds_vpd_c0(rp, len);
+            else if (VPD_VP_IBM_LTO == op->vend_prod_num)
+                decode_ibm_lto_dcrl(rp, len);
+            else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc1:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_SEAGATE == op->vend_prod_num)
+                decode_date_code_vpd_c1_sea(rp, len);
+            else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_c1(rp, len);
+            else if (VPD_VP_IBM_LTO == op->vend_prod_num)
+                decode_ibm_lto_dsn(rp, len);
+            else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc2:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num) {
+                if (as_json)
+                    jo2p = sg_vpd_js_hdr(jsp, jop,
+                                         "Software version VPD page", rp);
+                decode_rdac_vpd_c2(rp, len, op, jo2p);
+            } else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc3:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_SEAGATE == op->vend_prod_num)
+                decode_dev_beh_vpd_c3_sea(rp, len);
+            else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_c3(rp, len);
+            else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc4:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_c4(rp, len);
+            else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc5:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_HP_LTO == op->vend_prod_num)
+                decode_hp_lto_vpd_cx(rp, len, pn);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc8:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_c8(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xc9:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num) {
+                if (as_json)
+                    jo2p = sg_vpd_js_hdr(jsp, jop,
+                                         "Volume access control VPD page", rp);
+                decode_rdac_vpd_c9(rp, len, op, jo2p);
+            } else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xca:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_ca(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xd0:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_RDAC == op->vend_prod_num)
+                decode_rdac_vpd_d0(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xd1:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_WDC_HITACHI == op->vend_prod_num)
+                decode_vpd_d1_hit(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case 0xd2:
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+            } else if (VPD_VP_WDC_HITACHI == op->vend_prod_num)
+                decode_vpd_d2_hit(rp, len);
+            else
+                res = SG_LIB_CAT_OTHER;
+            break;
+        case SG_NVME_VPD_NICR:          /* 0xde */
+            if (dhex > 0) {
+		if (dhex > 2)
+		    named_hhh_output(NULL, rp, len, op);
+		else
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+                break;
+            } else if (VPD_VP_SG != op->vend_prod_num) {
+                res = SG_LIB_CAT_OTHER;
+                break;
             }
-            return 0;
+            /* NVMe: Identify Controller data structure (CNS 01h) */
+            plen = sg_get_unaligned_be16(rp + 2) + 4;
+            if (plen > len) {   /* fetch the whole page */
+                res = vpd_fetch_page(ptvp, rp, pn, plen, op->do_quiet,
+				     op->verbose, &len);
+                if (res) {
+                    pr2serr("Vendor VPD page=0x%x  failed to fetch\n", pn);
+                    return res;
+                }
+            }
+            if (len < 16) {
+                pr2serr("%s expected to be > 15 bytes long (got: %d)\n",
+                        name, len);
+                break;
+            } else {
+                int n = len - 16;
+                const char * np = "NVMe Identify Controller Response VPD page";
+                /* NVMe: Identify Controller data structure (CNS 01h) */
+                const char * ep = "(sg3_utils)";
+
+                if (n > 4096) {
+                    pr2serr("NVMe Identify response expected to be "
+                            "<= 4096 bytes (got: %d)\n", n);
+                    break;
+                }
+                if (op->do_hex < 3)
+                    sgj_pr_hr(jsp, "VPD INQUIRY: %s %s\n", np, ep);
+                if (op->do_hex)
+                    hex2stdout(rp, len, no_ascii_4hex(op));
+                else if (jsp->pr_as_json) {
+                    jo2p = sg_vpd_js_hdr(jsp, jop, np, rp);
+                    sgj_js_nv_hex_bytes(jsp, jo2p, "response_bytes",
+                                        rp + 16, n);
+                } else
+                    hex2stdout(rp + 16, n, 1);
+            }
+            break;
+        default:
+            res = SG_LIB_CAT_OTHER;
         }
-    } else
-        pr2serr("Vendor VPD page=0x%x  failed to fetch", op->vpd_pn);
+    }
+    if (res && (op->verbose > 1))
+        pr2serr("%s: can't decode pn=0x%x, vend_prod_num=%d\n", __func__,
+                pn, op->vend_prod_num);
     return res;
 }
