@@ -1,7 +1,7 @@
 /* A utility program for copying files. Specialised for "files" that
  * represent devices that understand the SCSI command set.
  *
- * Copyright (C) 1999 - 2021 D. Gilbert and P. Allworth
+ * Copyright (C) 1999 - 2023 D. Gilbert and P. Allworth
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -22,7 +22,7 @@
  * in this case) are transferred to or from the sg device in a single SCSI
  * command.
  *
- * This version is designed for the linux kernel 2.4, 2.6, 3, 4 and 5 series.
+ * This version is designed for the Linux kernel 2.4, 2.6, 3, 4 and 5 series.
  *
  * sgp_dd is a Posix threads specialization of the sg_dd utility. Both
  * sgp_dd and sg_dd only perform special tasks when one or both of the given
@@ -43,6 +43,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
+#include <time.h>               /* for clock_gettime() */
 #include <limits.h>
 #include <pthread.h>
 #include <signal.h>
@@ -56,13 +57,18 @@
 #include <sys/types.h>
 #endif
 #include <sys/time.h>
-#include <linux/major.h>        /* for MEM_MAJOR, SCSI_GENERIC_MAJOR, etc */
-#include <linux/fs.h>           /* for BLKSSZGET and friends */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef HAVE_LINUX_MAJOR_H
+#include <linux/major.h>
+#include <linux/fs.h>           /* for BLKSSZGET and friends */
+#else
+#include "sg_pt_linux_missing.h"
+#endif
+
 #ifdef __STDC_VERSION__
 #if __STDC_VERSION__ >= 201112L && defined(HAVE_STDATOMIC_H)
 #ifndef __STDC_NO_ATOMICS__
@@ -74,9 +80,12 @@
 #endif
 #endif
 
+#if 0
+/* The following warning produces a warning itself pre c++23 and c23 */
 #ifndef HAVE_C11_ATOMICS
 #warning "Don't have C11 Atomics, using mutex with pack_id"
 #endif
+#endif
 
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
@@ -85,13 +94,15 @@
 #include "sg_pr2serr.h"
 
 
-static const char * version_str = "5.83 20211105";
+static const char * version_str = "5.93 20230717";
 
 #define DEF_BLOCK_SIZE 512
 #define DEF_BLOCKS_PER_TRANSFER 128
 #define DEF_BLOCKS_PER_2048TRANSFER 32
 #define DEF_SCSI_CDBSZ 10
 #define MAX_SCSI_CDBSZ 16
+#define MAX_BPT_VALUE (1 << 24)         /* used for maximum bs as well */
+#define MAX_COUNT_SKIP_SEEK (1LL << 48) /* coverity wants upper bound */
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -171,7 +182,7 @@ struct opts_t
     bool mmap_active;
     int chkaddr;        /* check read data contains 4 byte, big endian block
                          * addresses, once: check only 4 bytes per block */
-    int progress;       /* accept --progress or -p, does nothing */
+    int progress;       /* --progress or -p, checked in sig_listen_thread */
     int debug;
     int dry_run;
 };
@@ -212,7 +223,7 @@ typedef struct request_element
 static sigset_t signal_set;
 static pthread_t sig_listen_thread_id;
 
-static const char * proc_allow_dio = "/proc/scsi/sg/allow_dio";
+static const char * sg_allow_dio = "/sys/module/sg/parameters/allow_dio";
 
 static void sg_in_operation(struct opts_t * clp, Rq_elem * rep);
 static void sg_out_operation(struct opts_t * clp, Rq_elem * rep,
@@ -223,6 +234,7 @@ static void normal_out_operation(struct opts_t * clp, Rq_elem * rep,
                                  int blocks, bool bump_out_blk);
 static int sg_start_io(Rq_elem * rep);
 static int sg_finish_io(bool wr, Rq_elem * rep, pthread_mutex_t * a_mutp);
+static bool check_progress(struct opts_t * clp);
 
 #ifdef HAVE_C11_ATOMICS
 
@@ -266,6 +278,7 @@ static struct thread_arg thr_arg_a[MAX_NUM_THREADS];
 static bool shutting_down = false;
 static bool do_sync = false;
 static bool do_time = false;
+static bool start_tm_valid = false;
 static struct opts_t my_opts;
 static struct timeval start_tm;
 static int64_t dd_count = -1;
@@ -276,8 +289,100 @@ static char outfn[INOUTF_SZ];
 static const char * my_name = "sgp_dd: ";
 
 
+/* Note that duration measurements may be effected by "discontinuous jumps
+ * in the system time". */
 static void
-calc_duration_throughput(int contin)
+calc_duration_throughput(bool contin)
+{
+    int n, blk_sz, elapsed_secs;
+    int64_t blks;
+    double a, b, r, da, db;
+    char f[128];
+    struct timeval end_tm, res_tm, delta_tm;
+    static const int flen = sizeof(f);
+    static bool prev_valid = false;
+    static struct timeval prev_tm;
+    static int64_t prev_blks;
+
+    f[0] = '\0';
+    if (start_tm_valid && (start_tm.tv_sec || start_tm.tv_usec)) {
+        blks = dd_count - my_opts.out_rem_count;
+        blk_sz = my_opts.bs;
+        gettimeofday(&end_tm, NULL);
+        res_tm.tv_sec = end_tm.tv_sec - start_tm.tv_sec;
+        res_tm.tv_usec = end_tm.tv_usec - start_tm.tv_usec;
+        if (res_tm.tv_usec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_usec += 1000000;
+        }
+        elapsed_secs = res_tm.tv_sec;
+        a = res_tm.tv_sec;
+        a += (0.000001 * res_tm.tv_usec);
+        if (prev_valid) {
+            delta_tm.tv_sec = end_tm.tv_sec - prev_tm.tv_sec;
+            delta_tm.tv_usec = end_tm.tv_usec - prev_tm.tv_usec;
+            if (delta_tm.tv_usec < 0) {
+                --delta_tm.tv_sec;
+                delta_tm.tv_usec += 1000000;
+            }
+            da = delta_tm.tv_sec;
+            da += (0.000001 * delta_tm.tv_usec);
+        } else
+            da = 0.0000001;
+
+        b = (double)blk_sz * blks;
+        n = sg_scnpr(f, flen, "time to copy data%s: %d.%06d secs",
+                     (contin ? " so far" : ""),
+                     (int)res_tm.tv_sec, (int)res_tm.tv_usec);
+        r = 0.0;
+        if ((a > 0.00001) && (b > 511)) {
+            r = b / (a * 1000000.0);
+            if (r < 1.0)
+                n += sg_scn3pr(f, flen, n, " at %.1f kB/sec", r * 1000);
+            else
+                n += sg_scn3pr(f, flen, n, " at %.2f MB/sec", r);
+        }
+        if (prev_valid && (da > 0.00001)) {
+            db = (double)blk_sz * (blks - prev_blks);
+            if (db > 511) {
+                double dr = db / (da * 1000000.0);
+
+                if (dr < 1.0)
+                    sg_scn3pr(f, flen, n, " (delta %.1f KB/sec)", dr * 1000);
+                else
+                    sg_scn3pr(f, flen, n, " (delta %.2f MB/sec)", dr);
+            }
+        }
+        pr2serr("%s\n", f);
+        if (contin && (r > 0.01) && (dd_count > 100)) {
+            int secs = (int)(((double)blk_sz * dd_count) / (r * 1000000));
+            int h, m;
+
+            if (secs > 10) {
+                n = sg_scnpr(f, flen, "%d%% complete, ",
+                             (100 * elapsed_secs) / (secs + elapsed_secs));
+                h = secs / 3600;
+                secs = secs - (h * 3600);
+                m = secs / 60;
+                secs = secs - (m * 60);
+                n += sg_scn3pr(f, flen, n, "estimated time remaining: ");
+                if (h > 0)
+                    sg_scn3pr(f, flen, n, "%d:%02d:%02d", h, m, secs);
+                else
+                    sg_scn3pr(f, flen, n, "%d:%02d", m, secs);
+                pr2serr("%s\n", f);
+            }
+        }
+        prev_tm = end_tm;
+        prev_blks = blks;
+        if (! prev_valid)
+            prev_valid = true;
+    }
+}
+
+#if 0
+static void
+calc_duration_throughput(bool contin)
 {
     struct timeval end_tm, res_tm;
     double a, b;
@@ -300,6 +405,7 @@ calc_duration_throughput(int contin)
     else
         pr2serr("\n");
 }
+#endif
 
 static void
 print_stats(const char * str)
@@ -329,7 +435,7 @@ interrupt_handler(int sig)
     sigaction(sig, &sigact, NULL);
     pr2serr("Interrupted by signal,");
     if (do_time)
-        calc_duration_throughput(0);
+        calc_duration_throughput(false);
     print_stats("");
     kill(getpid (), sig);
 }
@@ -340,7 +446,7 @@ siginfo_handler(int sig)
     if (sig) { ; }      /* unused, dummy to suppress warning */
     pr2serr("Progress report, continuing ...\n");
     if (do_time)
-        calc_duration_throughput(1);
+        calc_duration_throughput(true);
     print_stats("  ");
 }
 
@@ -370,13 +476,15 @@ thread_exit_handler(int sig)
 static char *
 tsafe_strerror(int code, char * ebp)
 {
+    int status;
     char * cp;
 
-    pthread_mutex_lock(&strerr_mut);
+    status = pthread_mutex_lock(&strerr_mut);
+    if (0 != status) pr2serr("lock strerr_mut");
     cp = safe_strerror(code);
     strncpy(ebp, cp, STRERR_BUFF_LEN);
-    pthread_mutex_unlock(&strerr_mut);
-
+    status = pthread_mutex_unlock(&strerr_mut);
+    if (0 != status) pr2serr("unlock strerr_mut");
     ebp[STRERR_BUFF_LEN - 1] = '\0';
     return ebp;
 }
@@ -386,9 +494,9 @@ tsafe_strerror(int code, char * ebp)
  * ISBN 0-201-63392-2 . [Highly recommended book.] Changed __FILE__
  * to __func__ */
 #define err_exit(code,text) do { \
-    char strerr_buff[STRERR_BUFF_LEN + 1]; \
+    char _strerr_buff[STRERR_BUFF_LEN + 1]; \
     pr2serr("%s at \"%s\":%d: %s\n", \
-        text, __func__, __LINE__, tsafe_strerror(code, strerr_buff)); \
+        text, __func__, __LINE__, tsafe_strerror(code, _strerr_buff)); \
     exit(1); \
     } while (0)
 
@@ -430,7 +538,7 @@ usage()
             "[deb=VERB] [dio=0|1]\n"
             "               [fua=0|1|2|3] [sync=0|1] [thr=THR] "
             "[time=0|1] [verbose=VERB]\n"
-            "               [--dry-run] [--verbose]\n"
+            "               [--dry-run] [--progress] [--verbose]\n"
             "  where:\n"
             "    bpt         is blocks_per_transfer (default is 128)\n"
             "    bs          must be device logical block size (default "
@@ -467,6 +575,7 @@ usage()
             "    --chkaddr|-c    check read data contains blk address\n"
             "    --dry-run|-d    prepare but bypass copy/read\n"
             "    --help|-h      output this usage message then exit\n"
+            "    --progress|-p    outputs progress report every 2 minutes\n"
             "    --verbose|-v   increase verbosity of utility\n"
             "    --version|-V   output version string then exit\n"
             "Copy from IFILE to OFILE, similar to dd command\n"
@@ -558,6 +667,7 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
     }
     return 0;
 #else
+    if (sg_fd) { ; }      /* unused, suppress warning */
     *num_sect = 0;
     *sect_sz = 0;
     return -1;
@@ -567,13 +677,26 @@ read_blkdev_capacity(int sg_fd, int64_t * num_sect, int * sect_sz)
 static void *
 sig_listen_thread(void * v_clp)
 {
-    struct opts_t * clp = (struct opts_t *)v_clp;
     int sig_number;
+    struct opts_t * clp = (struct opts_t *)v_clp;
+    struct timespec ts;
+    siginfo_t info;
 
-    while (1) {
-        sigwait(&signal_set, &sig_number);
+    ts.tv_sec = 0;
+    ts.tv_nsec = 200 * 1000 * 1000;
+
+    while (true) {
+        sig_number = sigtimedwait(&signal_set, &info, &ts);
         if (shutting_down)
             break;
+        if ((sig_number < 0) && (EAGAIN == errno)) {    /* time out */
+            if (clp->progress > 0) {
+                if (check_progress(clp)) {
+                    calc_duration_throughput(true);
+                    print_stats("");
+                }
+            }
+        }
         if (SIGINT == sig_number) {
             pr2serr("%sinterrupted by SIGINT\n", my_name);
 #ifdef HAVE_C11_ATOMICS
@@ -649,8 +772,10 @@ sg_in_open(const char * fnp, struct flags_t * flagp, int bs, int bpt)
         perror(ebuff);
         return -sg_convert_errno(err);
     }
-    if (sg_prepare(fd, bs, bpt))
+    if (sg_prepare(fd, bs, bpt)) {
+        close(fd);
         return -SG_LIB_FILE_ERROR;
+    }
     return fd;
 }
 
@@ -675,8 +800,10 @@ sg_out_open(const char * fnp, struct flags_t * flagp, int bs, int bpt)
         perror(ebuff);
         return -sg_convert_errno(err);
     }
-    if (sg_prepare(fd, bs, bpt))
+    if (sg_prepare(fd, bs, bpt)) {
+        close(fd);
         return -SG_LIB_FILE_ERROR;
+    }
     return fd;
 }
 
@@ -850,7 +977,6 @@ read_write_thread(void * v_tap)
         else
             normal_out_operation(clp, rep, blocks, enforce_write_ordering);
         pthread_cleanup_pop(0);
-
         if (enforce_write_ordering)
             pthread_cond_broadcast(&clp->out_sync_cv);
     } /* end of while loop */
@@ -1368,6 +1494,136 @@ num_chs_in_str(const char * s, int slen, int ch)
     return res;
 }
 
+#define PROGRESS_TRIGGER_MS 120000      /* milliseconds: 2 minutes */
+#define PROGRESS2_TRIGGER_MS 60000      /* milliseconds: 1 minute */
+#define PROGRESS3_TRIGGER_MS 30000      /* milliseconds: 30 seconds */
+
+/* Returns true when it time to output a progress report; else false. */
+static bool
+check_progress(struct opts_t * clp)
+{
+#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_MONOTONIC)
+    static bool have_prev, measure;
+    static struct timespec prev_true_tm;
+    static int count, threshold;
+    bool res = false;
+    uint32_t elapsed_ms, ms;
+    struct timespec now_tm, res_tm;
+
+    if (clp->progress) {
+        if (! have_prev) {
+            have_prev = true;
+            measure = true;
+            clock_gettime(CLOCK_MONOTONIC, &prev_true_tm);
+            return false;       /* starting reference */
+        }
+        if (! measure) {
+            if (++count >= threshold)
+                count = 0;
+            else
+                return false;
+        }
+        clock_gettime(CLOCK_MONOTONIC, &now_tm);
+        res_tm.tv_sec = now_tm.tv_sec - prev_true_tm.tv_sec;
+        res_tm.tv_nsec = now_tm.tv_nsec - prev_true_tm.tv_nsec;
+        if (res_tm.tv_nsec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_nsec += 1000000000;
+        }
+        elapsed_ms = (1000 * res_tm.tv_sec) + (res_tm.tv_nsec / 1000000);
+        if (measure) {
+            ++threshold;
+            if (elapsed_ms > 80)        /* 80 milliseconds */
+                measure = false;
+        }
+        if (elapsed_ms >= PROGRESS3_TRIGGER_MS) {
+            if (elapsed_ms >= PROGRESS2_TRIGGER_MS) {
+                if (elapsed_ms >= PROGRESS_TRIGGER_MS) {
+                    ms = PROGRESS_TRIGGER_MS;
+                    res = true;
+                } else if (clp->progress > 1) {
+                    ms = PROGRESS2_TRIGGER_MS;
+                    res = true;
+                }
+            } else if (clp->progress > 2) {
+                ms = PROGRESS3_TRIGGER_MS;
+                res = true;
+            }
+        }
+        if (res) {
+            prev_true_tm.tv_sec += (ms / 1000);
+            prev_true_tm.tv_nsec += (ms % 1000) * 1000000;
+            if (prev_true_tm.tv_nsec >= 1000000000) {
+                ++prev_true_tm.tv_sec;
+                prev_true_tm.tv_nsec -= 1000000000;
+            }
+        }
+    }
+    return res;
+
+#elif defined(HAVE_GETTIMEOFDAY)
+    static bool have_prev, measure;
+    static struct timeval prev_true_tm;
+    static int count, threshold;
+    bool res = false;
+    uint32_t elapsed_ms, ms;
+    struct timeval now_tm, res_tm;
+
+    if (clp->progress) {
+        if (! have_prev) {
+            have_prev = true;
+            gettimeofday(&prev_true_tm, NULL);
+            return false;       /* starting reference */
+        }
+        if (! measure) {
+            if (++count >= threshold)
+                count = 0;
+            else
+                return false;
+        }
+        gettimeofday(&now_tm, NULL);
+        res_tm.tv_sec = now_tm.tv_sec - prev_true_tm.tv_sec;
+        res_tm.tv_usec = now_tm.tv_usec - prev_true_tm.tv_usec;
+        if (res_tm.tv_usec < 0) {
+            --res_tm.tv_sec;
+            res_tm.tv_usec += 1000000;
+        }
+        elapsed_ms = (1000 * res_tm.tv_sec) + (res_tm.tv_usec / 1000);
+        if (measure) {
+            ++threshold;
+            if (elapsed_ms > 80)        /* 80 milliseconds */
+                measure = false;
+        }
+        if (elapsed_ms >= PROGRESS3_TRIGGER_MS) {
+            if (elapsed_ms >= PROGRESS2_TRIGGER_MS) {
+                if (elapsed_ms >= PROGRESS_TRIGGER_MS) {
+                    ms = PROGRESS_TRIGGER_MS;
+                    res = true;
+                } else if (clp->progress > 1) {
+                    ms = PROGRESS2_TRIGGER_MS;
+                    res = true;
+                }
+            } else if (clp->progress > 2) {
+                ms = PROGRESS3_TRIGGER_MS;
+                res = true;
+            }
+        }
+        if (res) {
+            prev_true_tm.tv_sec += (ms / 1000);
+            prev_true_tm.tv_usec += (ms % 1000) * 1000;
+            if (prev_true_tm.tv_usec >= 1000000) {
+                ++prev_true_tm.tv_sec;
+                prev_true_tm.tv_usec -= 1000000;
+            }
+        }
+    }
+    return res;
+
+#else   /* no clock reading functions available */
+    return false;
+#endif
+}
+
 
 int
 main(int argc, char * argv[])
@@ -1409,6 +1665,8 @@ main(int argc, char * argv[])
     clp->cdbsz_out = DEF_SCSI_CDBSZ;
     infn[0] = '\0';
     outfn[0] = '\0';
+    if (getenv("SG3_UTILS_INVOCATION"))
+        sg_rep_invocation(my_name, version_str, argc, argv, stderr);
 
     for (k = 1; k < argc; k++) {
         if (argv[k]) {
@@ -1424,19 +1682,23 @@ main(int argc, char * argv[])
         keylen = strlen(key);
         if (0 == strcmp(key,"bpt")) {
             clp->bpt = sg_get_num(buf);
-            if (-1 == clp->bpt) {
+            if ((clp->bpt < 0) || (clp->bpt > MAX_BPT_VALUE)) {
                 pr2serr("%sbad argument to 'bpt='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
             bpt_given = 1;
         } else if (0 == strcmp(key,"bs")) {
             clp->bs = sg_get_num(buf);
-            if (-1 == clp->bs) {
+            if ((clp->bs < 0) || (clp->bs > MAX_BPT_VALUE)) {
                 pr2serr("%sbad argument to 'bs='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"cdbsz")) {
             clp->cdbsz_in = sg_get_num(buf);
+            if ((clp->cdbsz_in < 6) || (clp->cdbsz_in > 32)) {
+                pr2serr("%s'cdbsz' expects 6, 10, 12, 16 or 32\n", my_name);
+                return SG_LIB_SYNTAX_ERROR;
+            }
             clp->cdbsz_out = clp->cdbsz_in;
             cdbsz_given = 1;
         } else if (0 == strcmp(key,"coe")) {
@@ -1445,7 +1707,7 @@ main(int argc, char * argv[])
         } else if (0 == strcmp(key,"count")) {
             if (0 != strcmp("-1", buf)) {
                 dd_count = sg_get_llnum(buf);
-                if (-1LL == dd_count) {
+                if ((dd_count < 0) || (dd_count > MAX_COUNT_SKIP_SEEK)) {
                     pr2serr("%sbad argument to 'count='\n", my_name);
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -1464,7 +1726,7 @@ main(int argc, char * argv[])
                 clp->in_flags.fua = true;
         } else if (0 == strcmp(key,"ibs")) {
             ibs = sg_get_num(buf);
-            if (-1 == ibs) {
+            if ((ibs < 0) || (ibs > MAX_BPT_VALUE)) {
                 pr2serr("%sbad argument to 'ibs='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
@@ -1483,7 +1745,7 @@ main(int argc, char * argv[])
             }
         } else if (0 == strcmp(key,"obs")) {
             obs = sg_get_num(buf);
-            if (-1 == obs) {
+            if ((obs < 0) || (obs > MAX_BPT_VALUE)) {
                 pr2serr("%sbad argument to 'obs='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
@@ -1502,13 +1764,13 @@ main(int argc, char * argv[])
             }
         } else if (0 == strcmp(key,"seek")) {
             seek = sg_get_llnum(buf);
-            if (-1LL == seek) {
+            if ((seek < 0) || (seek > MAX_COUNT_SKIP_SEEK)) {
                 pr2serr("%sbad argument to 'seek='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (0 == strcmp(key,"skip")) {
             skip = sg_get_llnum(buf);
-            if (-1LL == skip) {
+            if ((skip < 0) || (skip > MAX_COUNT_SKIP_SEEK)) {
                 pr2serr("%sbad argument to 'skip='\n", my_name);
                 return SG_LIB_SYNTAX_ERROR;
             }
@@ -1571,12 +1833,14 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
     }
+    if (clp->progress > 0)
+        do_time = true;
 
 #ifdef DEBUG
     pr2serr("In DEBUG mode, ");
     if (verbose_given && version_given) {
         pr2serr("but override: '-vV' given, zero verbose and continue\n");
-        verbose_given = false;
+        /* verbose_given = false;       */
         version_given = false;
         clp->debug = 0;
     } else if (! verbose_given) {
@@ -1611,8 +1875,8 @@ main(int argc, char * argv[])
         pr2serr("Can't use both append and seek switches\n");
         return SG_LIB_SYNTAX_ERROR;
     }
-    if (clp->bpt < 1) {
-        pr2serr("bpt must be greater than 0\n");
+    if ((clp->bpt < 1) || (clp->bpt > MAX_BPT_VALUE)) {
+        pr2serr("bpt must be > 0 and <= %d\n", MAX_BPT_VALUE);
         return SG_LIB_SYNTAX_ERROR;
     }
     if (clp->in_flags.mmap && clp->out_flags.mmap) {
@@ -1851,9 +2115,14 @@ main(int argc, char * argv[])
     clp->out_count = dd_count;
     clp->out_rem_count = dd_count;
     clp->seek = seek;
-    clp->out_blk = seek;
     status = pthread_mutex_init(&clp->inout_mutex, NULL);
     if (0 != status) err_exit(status, "init inout_mutex");
+    status = pthread_mutex_lock(&clp->inout_mutex);
+    if (0 != status) err_exit(status, "lock inout_mutex");
+    clp->out_blk = seek;
+    status = pthread_mutex_unlock(&clp->inout_mutex);
+    if (0 != status) err_exit(status, "unlock inout_mutex");
+
     status = pthread_cond_init(&clp->out_sync_cv, NULL);
     if (0 != status) err_exit(status, "init out_sync_cv");
 
@@ -1873,7 +2142,10 @@ main(int argc, char * argv[])
         start_tm.tv_sec = 0;
         start_tm.tv_usec = 0;
         gettimeofday(&start_tm, NULL);
+        start_tm_valid = true;
     }
+    if (FT_DEV_NULL == clp->in_type)
+        goto degen;     /* corner case: if=/dev/null */
 
 /* vvvvvvvvvvv  Start worker threads  vvvvvvvvvvvvvvvvvvvvvvvv */
     if ((clp->out_rem_count > 0) && (clp->num_threads > 0)) {
@@ -1918,8 +2190,9 @@ main(int argc, char * argv[])
         }
     }   /* started worker threads and here after they have all exited */
 
+degen:
     if (do_time && (start_tm.tv_sec || start_tm.tv_usec))
-        calc_duration_throughput(0);
+        calc_duration_throughput(false);
 
     if (do_sync) {
         if (FT_SG == clp->out_type) {
@@ -1974,11 +2247,11 @@ fini:
 
         pr2serr(">> Direct IO requested but incomplete %d times\n",
                 clp->dio_incomplete_count);
-        if ((fd = open(proc_allow_dio, O_RDONLY)) >= 0) {
+        if ((fd = open(sg_allow_dio, O_RDONLY)) >= 0) {
             if (1 == read(fd, &c, 1)) {
                 if ('0' == c)
                     pr2serr(">>> %s set to '0' but should be set to '1' for "
-                            "direct IO\n", proc_allow_dio);
+                            "direct IO\n", sg_allow_dio);
             }
             close(fd);
         }
