@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2018-2021 D. Gilbert
+ *  Copyright (C) 2018-2022 D. Gilbert
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
@@ -60,7 +60,7 @@
  * later of the Linux sg driver.  */
 
 
-static const char * version_str = "Version: 1.20  20210406";
+static const char * version_str = "Version: 1.21  20220202";
 
 #define INQ_REPLY_LEN 128
 #define INQ_CMD_LEN 6
@@ -193,7 +193,7 @@ timespec_diff(const struct timespec *lhs_p, const struct timespec *rhs_p,
 }
 
 /* Returns 0 on success. */
-int timespec2str(char *buf, uint len, struct timespec *ts)
+int timespec2str(char *buf, unsigned int len, struct timespec *ts)
 {
     int ret;
     struct tm t;
@@ -635,7 +635,7 @@ do_mrqs(int sg_fd, int sg_fd2, int mrqs)
     struct sg_io_v4 * h4p;
     struct sg_io_v4 * mrq_h4p;
     struct sg_io_v4 mrq_h4;
-    uint8_t sense_buffer[SENSE_BUFFER_LEN];
+    uint8_t sense_buffer[SENSE_BUFFER_LEN] SG_C_CPP_ZERO_INIT;
     uint8_t inq_cdb[INQ_CMD_LEN] =      /* Device Id VPD page */
                                 {0x12, 0x1, 0x83, 0, INQ_REPLY_LEN, 0};
     uint8_t sdiag_cdb[SDIAG_CMD_LEN] =
@@ -786,7 +786,7 @@ main(int argc, char * argv[])
     sg_io_hdr_t rio_hdr;
     char ebuff[EBUFF_SZ];
     char dname[256];
-    uint8_t sense_buffer[MAX_Q_LEN][SENSE_BUFFER_LEN];
+    uint8_t sense_buffer[MAX_Q_LEN][SENSE_BUFFER_LEN] SG_C_CPP_ZERO_INIT;
     const char * second_fname = NULL;
     const char * cp;
     char * chp;
